<?php
/**
 * NagVis - this extension add NagVis maps to mediawiki pages
 *
 * original by Felipe Muñoz Brieva 24.10.2008
 *
 * Installation:
 *
 * To activate this extension, add the following into your LocalSettings.php file:
 * require_once('$IP/extensions/NagVis/NagVis.php');
 *
 * @ingroup Extensions
 * @author Felipe Muñoz Brieva <felipe@gestiononline.es>
 * @version 1.0
 * @link http://www.mediawiki.org/wiki/Extension:NagVis Documentation
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License 2.0 or later
 *
 * Usage:
 *  Use only one <NagVis>-tag per MediaWiki page.
 *
 *  <NagVis --arguments-- >Header Text</NagVis>
 *
 *  --arguments--:
 *
 *     showheader: Shows a map header with links (NagVis/Nagios) and a counter showing pending time to refresh NagVis maps
 *     urlnagvis:  Url of NagVis website
 *     map:        NagVis map name
 *
 *  IMPORTANT: Only can be used one <NagVis>-tag per mediawiki page
 *
 * Example:
 *
 * <NagVis map=MapName showheader=yes urlnagvis=http://NagVisServer/nagios/nagvis/nagvis/index.php>Tittle (Map)</NagVis>
 *
 */
 
if (!defined( 'MEDIAWIKI' ) ) {
	echo( "This is an extension to the MediaWiki package and cannot be run standalone.\n" );
	die( -1 );
}
 
$wgExtensionFunctions[] = "wfNagVisExtension";

# NagVis extension credits

$wgExtensionCredits['parserhook'][] = array(
        'name' => 'NagVis (version 1.0)',
	'version'      => '1.0',
        'author' => 'Felipe Muñoz Brieva (email: felipe@gestiononline.net)',
        'url' => '',
        'description' => 'Add NagVis/Nagios maps to mediawiki pages'
);

# NagVis extension 

function wfNagVisExtension() {
  global $wgParser;
 
  $wgParser->setHook( "NagVis", "renderNagVis" );
}
 
# NagVis parser

function renderNagVis( $input , $args, $parser ) {
  global $wgVersion, $wgTitle, $wgDBprefix;

  global $wgOut, $wgScriptPath; 

  // Wrapper wiki offset

  $wiki_offsetx='2-(document.getElementById("content").offsetLeft)';
  $wiki_offsety='2-(document.getElementById("content").offsetTop)';
 
  // ###### INVALIDATE CACHE ######

  $parser->disableCache();
 
  // ###### LOAD NAGVIS ######

  $attr = array();   

  $showHeader="yes";
  $urlNagVis="";
  $firstLoad=true;
 
  foreach( $args as $name => $value ){
    switch(strtolower(htmlspecialchars($name))){ 
      case 'map':
            $map=htmlspecialchars($value);
            break;
      case 'showheader':
            $showHeader=htmlspecialchars($value);
            break;
      case 'urlnagvis':
            $urlNagVis=htmlspecialchars($value);
            break;
    }
  }

  if (trim($input)==""){
    $headerText=$map;
  }
  else {
    $headerText=htmlspecialchars($input);
  }

// -------------------------------------------

require_once('includes/simple_html_dom.php');

$urlNagiosBase = 'http://'.str_replace('nagvis/nagvis/index.php','',str_replace('http://','',$urlNagVis)); 
$urlNagVisBase = 'http://'.str_replace('index.php','',str_replace('http://','',$urlNagVis)); 
$urlNagVisMap = $urlNagVisBase."index.php?map=".$map; 

$html=new simple_html_dom();

// Check: Exists NagVis URL? 

if(!urlExists($urlNagVis)){
     $output.="<br>";
     $output.="ERROR: (urlNagVis)<br><br>";
     $output.="  URL not found: $urlNagVis<br>";
     $output.="<br>";
     return $output;
  }

$html->load_file($urlNagVisMap);

$titulo=$html->find('title',0);

// Check: Is it a NagVis URL?

  $position = strrpos(strtolower($titulo->innertext),'nagvis');
  
  if ($position === false){
     $output.="<br>";
     $output.="ERROR: (urlNagVisBase)<br><br>";
     $output.="  $urlNagVis isn't a NagVis URL<br>";
     $output.="<br>";
     return $output;
  }

// Check: Exists NagVis map?

if(!urlExists($urlNagVisMap)){
     $output.="<br>";
     $output.="ERROR: (urlNagVisMap)<br><br>";
     $output.="  $urlNagVisMap isn't a NagVis map<br>";
     $output.="<br>";
     return $output;
  }

// Define rotationSwitch

$wgOut->addScript('<link id="rotationSwitch" onclick="switchRotation(this,\'Start Rotation\',\'Stop Rotation\');" href="#" style="visibility: hidden;"></a>');

// Styles ----------------------------------------------------------------------

$contentcsssheet=new simple_html_dom();

$styles=$html->find('style');

foreach ($styles as $ss)
{
  $ss=str_replace('/nagios/',$urlNagiosBase,$ss);
  $ss=str_replace('./includes/',$urlNagVisBase.'includes/',$ss);
  $ss=str_replace('); -->','',$ss);
  $ss=str_replace('</style>','',$ss);
  $a=strpos($ss,'@import');
  if ($a > 0){
     $csssheet=substr($ss,$a+12); 
     $contentcsssheet=new simple_html_dom();
     $contentcsssheet->load_file($csssheet); 
     $valor=str_replace('body, table, th, td','not_used',$contentcsssheet);
     $wgOut->addScript('<style type="text/css">');
     $wgOut->addScript($valor);
     $wgOut->addScript('</style>');
     $contentcsssheet->clear();
     unset($contentcsssheet);
  }
}

// Scripts ---------------------------------------------------------------------

// Scripts src -----------------------------------------------------------------

$scriptssrc=$html->find('script[src]');

foreach ($scriptssrc as $s)
{
   $tmpscript=$s->src;
   $tmpscript=str_replace('./includes/',$urlNagVisBase.'includes/',$tmpscript);

   $file1=file($tmpscript);

   $wgOut->addScript('<script language="JavaScript" type="text/javascript">');

   foreach($file1 as $ss)
     {
       $ss=str_replace('getRequest(\'./ajax_handler.php?action=getObjectHoverMenu','getRequest(\'/wiki/extensions/NagVis/includes/ajax_handler_bridge.php?action=getObjectHoverMenu&urlNagVisBase='.$urlNagVisBase,$ss);
       $ss=str_replace('o3_offsetx=ol_offsetx;',"o3_offsetx=$wiki_offsetx;",$ss);
       $ss=str_replace('o3_offsety=ol_offsety;',"o3_offsety=$wiki_offsety;",$ss);
       $wgOut->addScript($ss);
     }

     $wgOut->addScript('</script>');
    
}

// Scripts language ------------------------------------------------------------

$scriptslanguage=$html->find('script[language]');

foreach ($scriptslanguage as $s)
{
  
   $wgOut->addScript($s);
}

// Header for NagVis map

 if ($showHeader=="yes"){
    $output.='<table class=header_table>';
    $output.='<tbody>';
    $output.='<td id="refreshCounterHead" style="width: 20px; text-align: center;">0</td>';
   
    $output.='<td style="width: 25px; text-align: center;">';
    $output.='<a href='.$urlNagVisMap.'>';
    $output.='<img alt="Nagios" src='.$wgScriptPath.'/extensions/NagVis/images/nagvis.png>';
    $output.='</a>';
    $output.='</td>';

    $output.='<td style="width: 25px; text-align: center;">';
    $output.='<a href='.$urlNagiosBase.'>';
    $output.='<img alt="Nagios" src='.$wgScriptPath.'/extensions/NagVis/images/nagios.png>';
    $output.='</a>';
    $output.='</td>';
    $output.='<td style="text-align: center;">';
    $output.='<a style="color: black;">'.$headerText.'</a>';
    $output.='</td>';

    $output.='</tbody>';
    $output.='</table>';
 }
 
 $divoverdiv=$html->find('div[id=overDiv]',0);
 $output.=$divoverdiv;
 
 $mapa=$html->find('div.map',0);
 $s = $mapa."";
 $s=str_replace('/nagios/',$urlNagiosBase,$s);
 $output.=$s;

// -------------------------------------------

  $html->clear();
  unset($html);
 
  return $output;
}

// Check : Exists URL ?

function urlExists($url) {
    $hdrs = @get_headers($url);
    return is_array($hdrs) ? preg_match('/^HTTP\\/\\d+\\.\\d+\\s+2\\d\\d\\s+.*$/',$hdrs[0]) : false;
}

?>
